// main.c
/********************************************************************/
/* ServoBot		                               	    	    */
/* with MSP430F149 @ 8MHz	                                    */                                      
/* Thomas Wedemeyer                                                 */
/* msp430@thomas-wedemeyer.de					    */
/*								    */
/* Some functions based on the MSPGCC-Examples from		    */
/* M.Mitchell (Texas Instruments, Inc) & Steve Underwood 	    */
/********************************************************************/

/*------------------------------------------------------------------------
	History:		von
	14.08.04		Thomas Wedemeyer
/*----------------------------------------------------------------------*/

/*------------------------------------------------------------------------
	Laden der Include-Files
/*----------------------------------------------------------------------*/
#include    <signal.h>
#include    <io.h>
#include    <stdio.h>
#include    "define.h"


struct  system_format
{
    volatile unsigned int    ticks;
    volatile unsigned char   new_tick;
};

struct system_format sys;
const char *array="Dies ist ein langer String...\r\n";
char    buffer[40];

/*----------------------------------------------------------------------*/
//!	Setup of the Clock XTAL2
/*----------------------------------------------------------------------*/
void set_clocks()
{
    unsigned int i;

    BCSCTL1 &= ~XT2OFF;                   // XT2 = HF XTAL

    do 
    {
        IFG1 &= ~OFIFG;                       // Clear OSCFault flag
        for (i = 0xFF; i > 0; i--);           // Time for flag to set
    }
    while ((IFG1 & OFIFG) != 0);          // OSCFault flag still set? 

    BCSCTL2 |= SELM1;                     // MCLK = XT2
    BCSCTL2 |= SELS;                      // SCLK = XTAL2
    BCSCTL2 &= ~(DIVM1 | DIVM0);          // DIV = 0
 
}

/*----------------------------------------------------------------------*/
//!	Init Timer B
/*----------------------------------------------------------------------*/
void init_pwm_timerb()
{
    // SMCLK, Clear Tar, SMCLK / 8 => tick = 1s, IRQ-Enable
    TBCTL = TBSSEL1 + TBCLR + ID0 + ID1 + TBIE;   
    TBCCR0 = 20000;                       // DutyCycle = 20ms 
    TBCCTL0 = OUTMOD_4;                   // Toggle als DebugPin...
    TBCCR1 = 1520;                        // 1,52ms
    TBCCTL1 = OUTMOD_7;                   // CCR1 reset/set
    TBCCR2 = 1520;                        // CCR1 PWM duty cycle
    TBCCTL2 = OUTMOD_7;                   // CCR2 reset/set
    P4DIR |= 0x07;                        // P1.2 and P1.3 output
    P4SEL |= 0x07;                        // P1.2 and P1.3 TA1/2 otions
    TBCTL |= MC0;                         // Start Timer_B in up mode
}

/*----------------------------------------------------------------------*/
//!	Timer_B7 Interrupt Vector (TBIV) handler
/*----------------------------------------------------------------------*/
interrupt (TIMERB1_VECTOR) Timer_B(void)
{
  switch( TBIV )
  {
   case  2: break;                      // CCR1 not used
   case  4: break;                      // CCR2 not used
   case 0x0E:                           // overflow
                sys.ticks++;
                sys.new_tick = 1;
                if(sys.ticks % 15)
                    P5OUT ^= 0x01;                      // Toggle P1.0 using exclusive-OR    
           break;
  }
}

/*----------------------------------------------------------------------*/
//!	Init of USART0
/*----------------------------------------------------------------------*/
void init_usart0()
{
    UCTL0 = CHAR;                         // 8-bit character
    UTCTL0 = SSEL_SMCLK;                  // UCLK = SMCLK
    UBR00 = 0xA1;                         // 8Mhz/19200 => 416,6666
    UBR10 = 0x01;                         //
    UMCTL0 = 0x00;                        // modulation
    ME1 |= UTXE0 + URXE0;                 // Enable USART0 TXD/RXD
    IE1 |= URXIE0;                        // Enable USART0 RX interrupt
    P3SEL |= 0x30;                        // P3.4,5 = USART1 option select
    P3DIR |= 0x18;                        // P3.4, P3.3 = output direction
    P3OUT |= 0x08;                        // Enable MAX3222CA
}

/*----------------------------------------------------------------------*/
//!	USART0 Receive Interrupt handler
/*----------------------------------------------------------------------*/
interrupt (UART0RX_VECTOR) usart0_rx(void)
{
 
    while ((IFG1 & UTXIFG0) == 0);        // USART1 TX buffer ready?
    TXBUF0 = RXBUF0;                      // RXBUF1 to TXBUF1

    //Workarround fr Platinenversion 1.0
    while ((IFG2 & UTXIFG1) == 0);        // USART1 TX buffer ready?
    TXBUF1 = RXBUF0;                      // RXBUF0 to TXBUF1
 
}

// Workarround fr Bug auf Platinen-Version 1.0 
/*----------------------------------------------------------------------*/
//!	Init of USART1
/*----------------------------------------------------------------------*/
void init_usart1()
{
    UCTL1 = CHAR;                         // 8-bit character
    UTCTL1 = SSEL_SMCLK;                  // UCLK = SMCLK
    UBR01 = 0xA1;                         // 8Mhz/19200 => 416,6666
    UBR11 = 0x01;                         //
    UMCTL1 = 0x00;                        // modulation
    ME2 |= UTXE1;                         // Enable USART1 TXD
    P3SEL |= 0xC0;                        // P3.4,5 = USART1 option select
    P3DIR |= 0x40;                        // P3.6 output direction
}


/*----------------------------------------------------------------------*/
//!	USART1 Simple putchar-function without buffer and IRQ,s
/*----------------------------------------------------------------------*/
void usart1_putchar(char character)
{
     //Workarround fr Platinenversion 1.0
    while ((IFG2 & UTXIFG1) == 0);        // USART1 TX buffer ready?
    TXBUF1 = character;                  // RXBUF0 to TXBUF1
}

/*----------------------------------------------------------------------*/
//!	USART1 Simple putstring-function without buffer and IRQ,s
/*----------------------------------------------------------------------*/
void usart1_puts(char *character)
{
    while(*character != 0)
    {
        usart1_putchar(*character);
        character++;
    }
}

/*----------------------------------------------------------------------*/
//!	Init of ADC12 Channel 0 & 1
/*----------------------------------------------------------------------*/
void init_adc12()
{
    ADC12CTL0 = ADC12ON + MSC+ SHT0_15;   // ADC on, Sampeltimer + Sampletime
    P6SEL |= 0x03;          // Analog Channel
    ADC12CTL1 = SHP+CONSEQ_1;               // Use sampling timer, single sequence
    ADC12MCTL0 = INCH_0;                    // ref+=AVcc, channel = A0
    ADC12MCTL1 = INCH_1+EOS;                // ref+=AVcc, channel = A1, end seq.
    ADC12CTL0 |= ENC;                     // Enable conversions
}

/*----------------------------------------------------------------------*/
//!	Read ADC (Polling)
/*----------------------------------------------------------------------*/
void get_adc12(unsigned int * data)
{
    ADC12CTL0 |= ADC12SC;                 // Start conversion
    while ((ADC12CTL1 & ADC12BUSY) == 1);   // ADC12BUSY?
    *data = ADC12MEM0;               // Move results, IFG is cleared
    data++;
    *data = ADC12MEM1;        
}
    

/*----------------------------------------------------------------------*/
//!	Auswertung der Bumper
/*----------------------------------------------------------------------*/
unsigned char check_bumper()
{
    unsigned char tmp=0;
    unsigned char workarround=0;

    tmp = ~P2IN; // Read Bumper
    // Ein bisschen Bitsdrehen, um den Ltkolben Kalt zu lassen...
      
    workarround = (tmp & 0x18) >> 1;
    if(workarround == 0x08)
        workarround = 0x04;
    else if (workarround == 0x04)
        workarround = 0x08;
    
    tmp = (tmp & 0x03) | workarround;

    return(tmp);
}

/*----------------------------------------------------------------------*/
//!	Set Motorspeed / Pulse lenght...
/*----------------------------------------------------------------------*/
void set_speed(int left_offset, int right_offset )
{

    TBCCR1 = M_STOP - left_offset;
    TBCCR2 = M_STOP + right_offset;
}

/*----------------------------------------------------------------------*/
//!	Wartefunktion x-Mal 20ms Warten
/*----------------------------------------------------------------------*/
void wait_ticks(unsigned int ticks)
{
    unsigned int wait;
    
    wait = sys.ticks + ticks;
    while (wait > sys.ticks);
}

/*----------------------------------------------------------------------*/
//!	Main
/*----------------------------------------------------------------------*/
int main(void)
{
    unsigned int i;
    unsigned int speed = 0;
    unsigned int clock=0;
    unsigned char   bumpercode;
    unsigned int lightsensor[2]; 
    int duty_right=M_FORWARDS, duty_left=M_FORWARDS; 
    int dif_sensor = 0;

    WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
    P5DIR |= 0x07;                        //  output direction P5.2 .. P5.0
    P5SEL |= 0x00;                        // 
    P2DIR = 0x00;                         // P2 Inputs
    P2SEL = 0x00;                       
  
    set_clocks();                         // Setup of Clocks             
    init_pwm_timerb();                    // Init Timer B for Servo control
    init_usart0();                          // Init of USART0                        
    init_usart1();                          // Init of USART1 (Workarround for Bug in Board 1.0)
    init_adc12();
    _EINT();

    usart1_puts((char *)array);
    usart1_puts("ServoRoboter\r\n");    
        
    for (;;)                              // Just Blink
    {
        if((sys.ticks % 50) == 0)
        {
            clock++;
            uprintf(usart1_putchar,"%d\r\n",clock);
            get_adc12( lightsensor);
            uprintf(usart1_putchar,"ADC0: %d ADC1: %d \r\n",lightsensor[0],lightsensor[1]);
        }
    
        if ( (P5IN & 0x10) == 0)
        {
            P5OUT |= 0x02;
            speed += 2;
            TBCCR1 = 1522+ speed;
            TBCCR2 = 1522- speed;
        }

        if ( (P5IN & 0x08) == 0)
        {
            P5OUT &= ~0x02;
            speed -= 2;
            TBCCR1 = 1522 + speed;
            TBCCR2 = 1522 - speed;
        }                        

        if((sys.ticks % 10) & (sys.new_tick == 1))
        {
            sys.new_tick = 0;
            bumpercode = check_bumper();
            switch(bumpercode)
            {
                case BM_FREE:   // Go for it...
                                    
                                    get_adc12( lightsensor);    // Sensorenabfragen	
    
                                    if( (lightsensor[0]) > (lightsensor[1])) // Links mehr Licht
                                    {
                                        duty_right=M_FORWARDS + ((lightsensor[0]-lightsensor[1]) >> 3);
                                   	    duty_left= M_FORWARDS - ((lightsensor[0]-lightsensor[1]) >> 3);
                                    }
                                    else if ((lightsensor[0]) < (lightsensor[1] )) // Rechts mehr Licht
                                    {
                                	   duty_right=M_FORWARDS - ((lightsensor[1]-lightsensor[0]) >> 3);
                                       duty_left= M_FORWARDS + ((lightsensor[1]-lightsensor[0]) >> 3);
                                    }
                                	
                                	set_speed(duty_left,duty_right);
    
    /*                                uprintf(usart1_putchar,"ADC0: %d ADC1: %d Left: %d Right: %d\r\n"
                                                ,lightsensor[0]
                                                ,lightsensor[1]
                                                ,duty_left
                                                ,duty_right);
    */
    
                                break;
                                
                case BM_FRONT:  //
                                break;
                                
                case BM_LEFT:   // LEft Bumper
                                P5OUT &= 0x02;
                                set_speed ( M_BACKWARDSSLOW, M_BACKWARDSSLOW);
                                wait_ticks(50);
                                
                                set_speed ( M_FORWARDS, M_BACKWARDS);
                                while (check_bumper() != 0 );
                                wait_ticks(50);
                                
                                set_speed ( M_FORWARDS, M_FORWARDS);
                                wait_ticks(50);
                                P5OUT |= 0x02; 
                                break; 
    
                case BM_RIGHT:   // RIGHT Bumper
                                P5OUT &= 0x04;
                                set_speed ( M_BACKWARDSSLOW, M_BACKWARDSSLOW);
                                wait_ticks(50);
                                
                                set_speed ( M_BACKWARDS, M_FORWARDS);
                                while (check_bumper() != 0 );
                                wait_ticks(50);
                                
                                set_speed ( M_FORWARDS, M_FORWARDS);
                                wait_ticks(50);
                                P5OUT |= 0x04; 
                                break;                             
                                  
                case BM_LERI:   // LEft & Right Bumper
                                set_speed ( M_BACKWARDSSLOW, M_BACKWARDSSLOW);
                                while (check_bumper() != 0 );
                                      
                                wait_ticks(50);
                                
                                set_speed ( M_FORWARDS, M_BACKWARDS);
                                while (check_bumper() != 0 );
                                wait_ticks(50);
                                set_speed ( M_FORWARDS, M_FORWARDS);
                                wait_ticks(50);
                                break; 
            }
        }

    }  
  
  
  return 0;
}
